@@ -52,12 +52,12 @@ struct _EmpathyTpContactListPriv {
 	GossipContact        *user_contact;
 	gboolean              setup;
 
-	GossipTelepathyGroup *known;
 	GossipTelepathyGroup *publish;
 	GossipTelepathyGroup *subscribe;
 
 	GHashTable           *groups;
 	GHashTable           *contacts;
+	GList                *local_pending;
 
 	DBusGProxy           *aliasing_iface;
 	DBusGProxy           *avatars_iface;
@@ -67,7 +67,6 @@ struct _EmpathyTpContactListPriv {
 };
 
 typedef enum {
-	TP_CONTACT_LIST_TYPE_KNOWN,
 	TP_CONTACT_LIST_TYPE_PUBLISH,
 	TP_CONTACT_LIST_TYPE_SUBSCRIBE,
 	TP_CONTACT_LIST_TYPE_UNKNOWN,
@@ -103,7 +102,10 @@ static void                   tp_contact_list_add                      (EmpathyC
 static void                   tp_contact_list_remove                   (EmpathyContactList              *list,
 									GossipContact                   *contact,
 									const gchar                     *message);
-static GList *                tp_contact_list_get_contacts             (EmpathyContactList              *list);
+static GList *                tp_contact_list_get_members              (EmpathyContactList              *list);
+static GList *                tp_contact_list_get_local_pending        (EmpathyContactList              *list);
+static void                   tp_contact_list_remove_local_pending     (EmpathyTpContactList            *list,
+									GossipContact                   *contact);
 static void                   tp_contact_list_contact_removed_foreach  (guint                            handle,
 									GossipContact                   *contact,
 									EmpathyTpContactList            *list);
@@ -120,20 +122,20 @@ static void                   tp_contact_list_newchannel_cb            (DBusGPro
 									gboolean                         suppress_handle,
 									EmpathyTpContactList            *list);
 static TpContactListType      tp_contact_list_get_type                 (EmpathyTpContactList            *list,
-									TpChan                          *list_chan);
-static void                   tp_contact_list_contact_added_cb         (GossipTelepathyGroup            *group,
+									GossipTelepathyGroup            *group);
+static void                   tp_contact_list_added_cb                 (GossipTelepathyGroup            *group,
 									GArray                          *handles,
 									guint                            actor_handle,
 									guint                            reason,
 									const gchar                     *message,
 									EmpathyTpContactList            *list);
-static void                   tp_contact_list_contact_removed_cb       (GossipTelepathyGroup            *group,
+static void                   tp_contact_list_removed_cb               (GossipTelepathyGroup            *group,
 									GArray                          *handles,
 									guint                            actor_handle,
 									guint                            reason,
 									const gchar                     *message,
 									EmpathyTpContactList            *list);
-static void                   tp_contact_list_local_pending_cb         (GossipTelepathyGroup            *group,
+static void                   tp_contact_list_pending_cb               (GossipTelepathyGroup            *group,
 									GArray                          *handles,
 									guint                            actor_handle,
 									guint                            reason,
@@ -142,9 +144,6 @@ static void                   tp_contact_list_local_pending_cb         (GossipTe
 static void                   tp_contact_list_groups_updated_cb        (GossipContact                   *contact,
 									GParamSpec                      *param,
 									EmpathyTpContactList            *list);
-static void                   tp_contact_list_subscription_updated_cb  (GossipContact                   *contact,
-									GParamSpec                      *param,
-									EmpathyTpContactList            *list);
 static void                   tp_contact_list_name_updated_cb          (GossipContact                   *contact,
 									GParamSpec                      *param,
 									EmpathyTpContactList            *list);
@@ -173,7 +172,7 @@ static void                   tp_contact_list_group_members_removed_cb (GossipTe
 									guint                            reason,
 									const gchar                     *message,
 									EmpathyTpContactList            *list);
-static void                   tp_contact_list_get_contacts_foreach     (guint                            handle,
+static void                   tp_contact_list_get_members_foreach      (guint                            handle,
 									GossipContact                   *contact,
 									GList                          **contacts);
 static void                   tp_contact_list_get_info                 (EmpathyTpContactList            *list,
@@ -248,11 +247,12 @@ empathy_tp_contact_list_class_init (EmpathyTpContactListClass *klass)
 static void
 tp_contact_list_iface_init (EmpathyContactListIface *iface)
 {
-	iface->setup = tp_contact_list_setup;
-	iface->find = tp_contact_list_find;
-	iface->add = tp_contact_list_add;
-	iface->remove = tp_contact_list_remove;
-	iface->get_contacts = tp_contact_list_get_contacts;
+	iface->setup             = tp_contact_list_setup;
+	iface->find              = tp_contact_list_find;
+	iface->add               = tp_contact_list_add;
+	iface->remove            = tp_contact_list_remove;
+	iface->get_members       = tp_contact_list_get_members;
+	iface->get_local_pending = tp_contact_list_get_local_pending;
 }
 
 static void
@@ -293,15 +293,9 @@ tp_contact_list_finalize (GObject *object)
 	if (priv->tp_conn) {
 		g_object_unref (priv->tp_conn);
 	}
-
-	if (priv->known) {
-		g_object_unref (priv->known);
-	}
-
 	if (priv->subscribe) {
 		g_object_unref (priv->subscribe);
 	}
-
 	if (priv->publish) {
 		g_object_unref (priv->publish);
 	}
@@ -312,6 +306,9 @@ tp_contact_list_finalize (GObject *object)
 	g_hash_table_destroy (priv->groups);
 	g_hash_table_destroy (priv->contacts);
 
+	g_list_foreach (priv->local_pending, (GFunc) empathy_contact_list_info_free, NULL);
+	g_list_free (priv->local_pending);
+
 	G_OBJECT_CLASS (empathy_tp_contact_list_parent_class)->finalize (object);
 }
 
@@ -383,6 +380,7 @@ empathy_tp_contact_list_new (McAccount *account)
 			      error ? error->message : "No error given");
 		g_clear_error (&error);
 	} else {
+		/* FIXME: this adds the handle to the roster */
 		priv->user_contact = empathy_tp_contact_list_get_from_handle (list, handle);
 	}
 
@@ -473,6 +471,7 @@ tp_contact_list_add (EmpathyContactList *list,
 
 	handle = gossip_contact_get_handle (contact);
 	gossip_telepathy_group_add_member (priv->subscribe, handle, message);
+	gossip_telepathy_group_add_member (priv->publish, handle, message);
 }
 
 static void
@@ -490,11 +489,10 @@ tp_contact_list_remove (EmpathyContactList *list,
 	handle = gossip_contact_get_handle (contact);
 	gossip_telepathy_group_remove_member (priv->subscribe, handle, message);
 	gossip_telepathy_group_remove_member (priv->publish, handle, message);
-	gossip_telepathy_group_remove_member (priv->known, handle, message);
 }
 
 static GList *
-tp_contact_list_get_contacts (EmpathyContactList *list)
+tp_contact_list_get_members (EmpathyContactList *list)
 {
 	EmpathyTpContactListPriv *priv;
 	GList                    *contacts = NULL;
@@ -503,14 +501,25 @@ tp_contact_list_get_contacts (EmpathyContactList *list)
 
 	priv = GET_PRIV (list);
 
-	/* FIXME: we should only return contacts that are in the contact list */
 	g_hash_table_foreach (priv->contacts,
-			      (GHFunc) tp_contact_list_get_contacts_foreach,
+			      (GHFunc) tp_contact_list_get_members_foreach,
 			      &contacts);
 
 	return contacts;
 }
 
+static GList *
+tp_contact_list_get_local_pending (EmpathyContactList *list)
+{
+	EmpathyTpContactListPriv *priv;
+
+	g_return_val_if_fail (EMPATHY_IS_TP_CONTACT_LIST (list), NULL);
+
+	priv = GET_PRIV (list);
+
+	return g_list_copy (priv->local_pending);
+}
+
 McAccount *
 empathy_tp_contact_list_get_account (EmpathyTpContactList *list)
 {
@@ -553,7 +562,7 @@ empathy_tp_contact_list_get_from_id (EmpathyTpContactList *list,
 
 	contact = tp_contact_list_find (EMPATHY_CONTACT_LIST (list), id);
 	if (contact) {
-		return contact;
+		return g_object_ref (contact);
 	}
 
 	/* The id is unknown, requests a new handle */
@@ -565,7 +574,7 @@ empathy_tp_contact_list_get_from_id (EmpathyTpContactList *list,
 			      "RequestHandle for %s failed: %s", id,
 			      error ? error->message : "No error given");
 		g_clear_error (&error);
-		return 0;
+		return NULL;
 	}
 
 	handle = g_array_index(handles, guint, 0);
@@ -684,12 +693,21 @@ empathy_tp_contact_list_get_from_handles (EmpathyTpContactList *list,
 					"handle", handle,
 					NULL);
 
+		if (!priv->presence_iface) {
+			GossipPresence *presence;
+
+			/* We have no presence iface, set default presence
+			 * to available */
+			presence = gossip_presence_new_full (MC_PRESENCE_AVAILABLE,
+							     NULL);
+
+			gossip_contact_set_presence (contact, presence);
+			g_object_unref (presence);
+		}
+
 		g_signal_connect (contact, "notify::groups",
 				  G_CALLBACK (tp_contact_list_groups_updated_cb),
 				  list);
-		g_signal_connect (contact, "notify::subscription",
-				  G_CALLBACK (tp_contact_list_subscription_updated_cb),
-				  list);
 		g_signal_connect (contact, "notify::name",
 				  G_CALLBACK (tp_contact_list_name_updated_cb),
 				  list);
@@ -847,9 +865,6 @@ tp_contact_list_contact_removed_foreach (guint                 handle,
 	g_signal_handlers_disconnect_by_func (contact,
 					      tp_contact_list_groups_updated_cb,
 					      list);
-	g_signal_handlers_disconnect_by_func (contact,
-					      tp_contact_list_subscription_updated_cb,
-					      list);
 	g_signal_handlers_disconnect_by_func (contact,
 					      tp_contact_list_name_updated_cb,
 					      list);
@@ -864,9 +879,6 @@ tp_contact_list_block_contact (EmpathyTpContactList *list,
 	g_signal_handlers_block_by_func (contact,
 					 tp_contact_list_groups_updated_cb,
 					 list);
-	g_signal_handlers_block_by_func (contact,
-					 tp_contact_list_subscription_updated_cb,
-					 list);
 	g_signal_handlers_block_by_func (contact,
 					 tp_contact_list_name_updated_cb,
 					 list);
@@ -879,9 +891,6 @@ tp_contact_list_unblock_contact (EmpathyTpContactList *list,
 	g_signal_handlers_unblock_by_func (contact,
 					   tp_contact_list_groups_updated_cb,
 					   list);
-	g_signal_handlers_unblock_by_func (contact,
-					   tp_contact_list_subscription_updated_cb,
-					   list);
 	g_signal_handlers_unblock_by_func (contact,
 					   tp_contact_list_name_updated_cb,
 					   list);
@@ -931,88 +940,78 @@ tp_contact_list_newchannel_cb (DBusGProxy           *proxy,
 	if (handle_type == TP_HANDLE_TYPE_LIST) {
 		TpContactListType list_type;
 
-		list_type = tp_contact_list_get_type (list, new_chan);
-		if (list_type == TP_CONTACT_LIST_TYPE_UNKNOWN) {
-			gossip_debug (DEBUG_DOMAIN, "Unknown contact list channel");
-			g_object_unref (new_chan);
-			return;
-		}
-
-		gossip_debug (DEBUG_DOMAIN, "New contact list channel of type: %d",
-			      list_type);
-
 		group = gossip_telepathy_group_new (new_chan, priv->tp_conn);
 
-		switch (list_type) {
-		case TP_CONTACT_LIST_TYPE_KNOWN:
-			if (priv->known) {
-				g_object_unref (priv->known);
-			}
-			priv->known = group;
-			break;
-		case TP_CONTACT_LIST_TYPE_PUBLISH:
+		list_type = tp_contact_list_get_type (list, group);
+		if (list_type == TP_CONTACT_LIST_TYPE_UNKNOWN) {
+			gossip_debug (DEBUG_DOMAIN,
+				      "Type of contact list channel unknown: %s",
+				      gossip_telepathy_group_get_name (group));
+
+			g_object_unref (new_chan);
+			g_object_unref (group);
+			return;
+		} else {
+			gossip_debug (DEBUG_DOMAIN,
+				      "New contact list channel of type: %d",
+				      list_type);
+		}
+
+		g_signal_connect (group, "members-added",
+				  G_CALLBACK (tp_contact_list_added_cb),
+				  list);
+		g_signal_connect (group, "members-removed",
+				  G_CALLBACK (tp_contact_list_removed_cb),
+				  list);
+
+		members = gossip_telepathy_group_get_members (group);
+		tp_contact_list_added_cb (group, members, 0,
+					  TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
+					  NULL, list);
+		g_array_free (members, TRUE);
+
+		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			GList *pendings, *l;
+
 			if (priv->publish) {
 				g_object_unref (priv->publish);
 			}
 			priv->publish = group;
-			break;
-		case TP_CONTACT_LIST_TYPE_SUBSCRIBE:
+
+			/* Makes no sense to be in remote-pending */
+			g_signal_connect (group, "local-pending",
+					  G_CALLBACK (tp_contact_list_pending_cb),
+					  list);
+
+			pendings = gossip_telepathy_group_get_local_pending_members_with_info (group);
+			if (pendings) {
+				GArray *pending;
+
+				pending = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
+				for (l = pendings; l; l = l->next) {
+					GossipTpGroupInfo *info;
+
+					info = l->data;
+
+					g_array_insert_val (pending, 0, info->member);
+					tp_contact_list_pending_cb (group, pending,
+								    info->actor,
+								    info->reason,
+								    info->message,
+								    list);
+				}
+				g_array_free (pending, TRUE);
+				gossip_telepathy_group_info_list_free (pendings);
+			}
+		}
+		else if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE) {
 			if (priv->subscribe) {
 				g_object_unref (priv->subscribe);
 			}
 			priv->subscribe = group;
-			break;
-		default:
+		} else {
 			g_assert_not_reached ();
 		}
-
-		/* Connect and setup the new contact-list group */
-		if (list_type == TP_CONTACT_LIST_TYPE_KNOWN ||
-		    list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE) {
-			g_signal_connect (group, "members-added",
-					  G_CALLBACK (tp_contact_list_contact_added_cb),
-					  list);
-			g_signal_connect (group, "members-removed",
-					  G_CALLBACK (tp_contact_list_contact_removed_cb),
-					  list);
-
-			members = gossip_telepathy_group_get_members (group);
-			tp_contact_list_contact_added_cb (group, members, 0,
-						       TP_CHANNEL_GROUP_CHANGE_REASON_NONE,
-						       NULL, list);
-			g_array_free (members, TRUE);
-		}
-		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
-			GList  *members, *l;
-			GArray *pending;
-
-			g_signal_connect (group, "local-pending",
-					  G_CALLBACK (tp_contact_list_local_pending_cb),
-					  list);
-
-			members = gossip_telepathy_group_get_local_pending_members_with_info (group);
-			if (!members) {
-				g_object_unref (new_chan);
-				return;
-			}
-
-			pending = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
-			for (l = members; l; l = l->next) {
-				GossipTpGroupInfo *info;
-
-				info = l->data;
-
-				g_array_insert_val (pending, 0, info->member);
-				tp_contact_list_local_pending_cb (group, pending,
-								  info->actor,
-								  info->reason,
-								  info->message,
-								  list);
-			}
-
-			gossip_telepathy_group_info_list_free (members);
-			g_array_free (pending, TRUE);
-		}
 	}
 	else if (handle_type == TP_HANDLE_TYPE_GROUP) {
 		const gchar *object_path;
@@ -1053,72 +1052,69 @@ tp_contact_list_newchannel_cb (DBusGProxy           *proxy,
 
 static TpContactListType
 tp_contact_list_get_type (EmpathyTpContactList *list,
-			  TpChan               *list_chan)
+			  GossipTelepathyGroup *group)
 {
 	EmpathyTpContactListPriv  *priv;
-	GArray                    *handles;
-	gchar                    **handle_name;
 	TpContactListType          list_type;
-	GError                    *error = NULL;
+	const gchar               *name;
 
 	priv = GET_PRIV (list);
 
-	handles = g_array_new (FALSE, FALSE, sizeof (guint));
-	g_array_append_val (handles, list_chan->handle);
-
-	if (!tp_conn_inspect_handles (DBUS_G_PROXY (priv->tp_conn),
-				      TP_HANDLE_TYPE_LIST,
-				      handles,
-				      &handle_name,
-				      &error)) {
-		gossip_debug (DEBUG_DOMAIN, 
-			      "InspectHandle Error: %s",
-			      error ? error->message : "No error given");
-		g_clear_error (&error);
-		g_array_free (handles, TRUE);
-		return TP_CONTACT_LIST_TYPE_UNKNOWN;
-	}
-
-	if (strcmp (*handle_name, "subscribe") == 0) {
+	name = gossip_telepathy_group_get_name (group);
+	if (strcmp (name, "subscribe") == 0) {
 		list_type = TP_CONTACT_LIST_TYPE_SUBSCRIBE;
-	} else if (strcmp (*handle_name, "publish") == 0) {
+	} else if (strcmp (name, "publish") == 0) {
 		list_type = TP_CONTACT_LIST_TYPE_PUBLISH;
-	} else if (strcmp (*handle_name, "known") == 0) {
-		list_type = TP_CONTACT_LIST_TYPE_KNOWN;
 	} else {
 		list_type = TP_CONTACT_LIST_TYPE_UNKNOWN;
 	}
 
-	g_strfreev (handle_name);
-	g_array_free (handles, TRUE);
-
 	return list_type;
 }
 
 static void
-tp_contact_list_contact_added_cb (GossipTelepathyGroup *group,
-				  GArray               *handles,
-				  guint                 actor_handle,
-				  guint                 reason,
-				  const gchar          *message,
-				  EmpathyTpContactList *list)
+tp_contact_list_added_cb (GossipTelepathyGroup *group,
+			  GArray               *handles,
+			  guint                 actor_handle,
+			  guint                 reason,
+			  const gchar          *message,
+			  EmpathyTpContactList *list)
 {
 	EmpathyTpContactListPriv *priv;
 	GList                    *added_list, *l;
+	TpContactListType         list_type;
 
 	priv = GET_PRIV (list);
 
-	added_list = empathy_tp_contact_list_get_from_handles (list, handles);
+	list_type = tp_contact_list_get_type (list, group);
 
+	added_list = empathy_tp_contact_list_get_from_handles (list, handles);
 	for (l = added_list; l; l = l->next) {
-		GossipContact *contact;
+		GossipContact      *contact;
+		GossipSubscription  subscription;
 
 		contact = GOSSIP_CONTACT (l->data);
+
+		gossip_debug (DEBUG_DOMAIN, "Contact '%s' added to list type %d",
+			      gossip_contact_get_name (contact),
+			      list_type);
+
+		subscription = gossip_contact_get_subscription (contact);
+		if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE) {
+			subscription |= GOSSIP_SUBSCRIPTION_FROM;
+		}
+		else if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			subscription |= GOSSIP_SUBSCRIPTION_TO;
+		}
+
 		tp_contact_list_block_contact (list, contact);
-		gossip_contact_set_subscription (contact, GOSSIP_SUBSCRIPTION_BOTH);
+		gossip_contact_set_subscription (contact, subscription);
 		tp_contact_list_unblock_contact (list, contact);
 
-		g_signal_emit_by_name (list, "contact-added", contact);
+		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			tp_contact_list_remove_local_pending (list, contact);
+			g_signal_emit_by_name (list, "contact-added", contact);
+		}
 
 		g_object_unref (contact);
 	}
@@ -1127,31 +1123,48 @@ tp_contact_list_contact_added_cb (GossipTelepathyGroup *group,
 }
 
 static void
-tp_contact_list_contact_removed_cb (GossipTelepathyGroup *group,
-				    GArray               *handles,
-				    guint                 actor_handle,
-				    guint                 reason,
-				    const gchar          *message,
-				    EmpathyTpContactList *list)
+tp_contact_list_removed_cb (GossipTelepathyGroup *group,
+			    GArray               *handles,
+			    guint                 actor_handle,
+			    guint                 reason,
+			    const gchar          *message,
+			    EmpathyTpContactList *list)
 {
 	EmpathyTpContactListPriv *priv;
 	GList                    *removed_list, *l;
+	TpContactListType         list_type;
 
 	priv = GET_PRIV (list);
 
-	removed_list = empathy_tp_contact_list_get_from_handles (list, handles);
+	list_type = tp_contact_list_get_type (list, group);
 
+	removed_list = empathy_tp_contact_list_get_from_handles (list, handles);
 	for (l = removed_list; l; l = l->next) {
-		GossipContact *contact;
-		guint          handle;
+		GossipContact      *contact;
+		GossipSubscription  subscription;
 
 		contact = GOSSIP_CONTACT (l->data);
 
-		handle = gossip_contact_get_handle (contact);
-		g_hash_table_remove (priv->contacts, GUINT_TO_POINTER (handle));
+		gossip_debug (DEBUG_DOMAIN, "Contact '%s' removed from list type %d",
+			      gossip_contact_get_name (contact),
+			      list_type);
 
-		g_signal_emit_by_name (list, "contact-removed", contact);
+		subscription = gossip_contact_get_subscription (contact);
+		if (list_type == TP_CONTACT_LIST_TYPE_SUBSCRIBE) {
+			subscription &= !GOSSIP_SUBSCRIPTION_FROM;
+		}
+		else if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			subscription &= !GOSSIP_SUBSCRIPTION_TO;
+		}
 
+		tp_contact_list_block_contact (list, contact);
+		gossip_contact_set_subscription (contact, subscription);
+		tp_contact_list_unblock_contact (list, contact);
+
+		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			tp_contact_list_remove_local_pending (list, contact);
+			g_signal_emit_by_name (list, "contact-removed", contact);
+		}
 		g_object_unref (contact);
 	}
 
@@ -1159,30 +1172,41 @@ tp_contact_list_contact_removed_cb (GossipTelepathyGroup *group,
 }
 
 static void
-tp_contact_list_local_pending_cb (GossipTelepathyGroup *group,
-				  GArray               *handles,
-				  guint                 actor_handle,
-				  guint                 reason,
-				  const gchar          *message,
-				  EmpathyTpContactList *list)
+tp_contact_list_pending_cb (GossipTelepathyGroup *group,
+			    GArray               *handles,
+			    guint                 actor_handle,
+			    guint                 reason,
+			    const gchar          *message,
+			    EmpathyTpContactList *list)
 {
 	EmpathyTpContactListPriv *priv;
 	GList                    *pending_list, *l;
+	TpContactListType         list_type;
 
 	priv = GET_PRIV (list);
 
-	pending_list = empathy_tp_contact_list_get_from_handles (list, handles);
+	list_type = tp_contact_list_get_type (list, group);
 
+	pending_list = empathy_tp_contact_list_get_from_handles (list, handles);
 	for (l = pending_list; l; l = l->next) {
 		GossipContact *contact;
 
 		contact = GOSSIP_CONTACT (l->data);
 
-		/* FIXME: Is that the correct way ? */
-		tp_contact_list_block_contact (list, contact);
-		gossip_contact_set_subscription (contact, GOSSIP_SUBSCRIPTION_FROM);
-		tp_contact_list_unblock_contact (list, contact);
-		g_signal_emit_by_name (list, "contact-added", contact);
+		gossip_debug (DEBUG_DOMAIN, "Contact '%s' pending in list type %d",
+			      gossip_contact_get_name (contact),
+			      list_type);
+
+		if (list_type == TP_CONTACT_LIST_TYPE_PUBLISH) {
+			EmpathyContactListInfo *info;
+
+			info = empathy_contact_list_info_new (contact, message);
+			priv->local_pending = g_list_prepend (priv->local_pending,
+							      info);
+
+			g_signal_emit_by_name (list, "local-pending",
+					       contact, message);
+		}
 
 		g_object_unref (contact);
 	}
@@ -1190,6 +1214,30 @@ tp_contact_list_local_pending_cb (GossipTelepathyGroup *group,
 	g_list_free (pending_list);
 }
 
+static void
+tp_contact_list_remove_local_pending (EmpathyTpContactList *list,
+				      GossipContact        *contact)
+{
+	EmpathyTpContactListPriv *priv;
+	GList                    *l;
+
+	priv = GET_PRIV (list);
+
+	for (l = priv->local_pending; l; l = l->next) {
+		EmpathyContactListInfo *info;
+
+		info = l->data;
+		if (gossip_contact_equal (contact, info->contact)) {
+			gossip_debug (DEBUG_DOMAIN, "Contact no more local-pending: %s",
+				      gossip_contact_get_name (contact));
+
+			priv->local_pending = g_list_delete_link (priv->local_pending, l);
+			empathy_contact_list_info_free (info);
+			break;
+		}
+	}
+}
+
 static void
 tp_contact_list_groups_updated_cb (GossipContact        *contact,
 				   GParamSpec           *param,
@@ -1215,28 +1263,6 @@ tp_contact_list_groups_updated_cb (GossipContact        *contact,
 			      &data);
 }
 
-static void
-tp_contact_list_subscription_updated_cb (GossipContact        *contact,
-					 GParamSpec           *param,
-					 EmpathyTpContactList *list)
-{
-	EmpathyTpContactListPriv *priv;
-	GossipSubscription        subscription;
-	guint                     handle;
-
-	priv = GET_PRIV (list);
-
-	subscription = gossip_contact_get_subscription (contact);
-	handle = gossip_contact_get_handle (contact);
-
-	/* FIXME: what to do here, I'm a bit lost... */
-	if (subscription) {
-		gossip_telepathy_group_add_member (priv->publish, handle, "");
-	} else {
-		gossip_telepathy_group_remove_member (priv->publish, handle, "");
-	}
-}
-
 static void
 tp_contact_list_name_updated_cb (GossipContact        *contact,
 				 GParamSpec           *param,
@@ -1444,24 +1470,12 @@ tp_contact_list_group_members_added_cb (GossipTelepathyGroup *group,
 
 	for (l = added_list; l; l = l->next) {
 		GossipContact *contact;
-		GList         *contact_groups;
 
 		contact = GOSSIP_CONTACT (l->data);
-		contact_groups = gossip_contact_get_groups (contact);
 
-		/* FIXME: this leaks */
-		if (!g_list_find_custom (contact_groups,
-					 group_name,
-					 (GCompareFunc) strcmp)) {
-			gossip_debug (DEBUG_DOMAIN, "Contact %s added to group '%s'",
-				      gossip_contact_get_name (contact),
-				      group_name);
-			contact_groups = g_list_append (contact_groups,
-							g_strdup (group_name));
-			tp_contact_list_block_contact (list, contact);
-			gossip_contact_set_groups (contact, contact_groups);
-			tp_contact_list_unblock_contact (list, contact);
-		}
+		tp_contact_list_block_contact (list, contact);
+		gossip_contact_add_group (contact, group_name);
+		tp_contact_list_unblock_contact (list, contact);
 
 		g_object_unref (contact);
 	}
@@ -1488,29 +1502,12 @@ tp_contact_list_group_members_removed_cb (GossipTelepathyGroup *group,
 
 	for (l = removed_list; l; l = l->next) {
 		GossipContact *contact;
-		GList         *contact_groups;
-		GList         *to_remove;
 
-		/* FIXME: Does it leak ? */
-		contact = GOSSIP_CONTACT (l->data);
-		contact_groups = gossip_contact_get_groups (contact);
-		contact_groups = g_list_copy (contact_groups);
+		contact = l->data;
 
-		to_remove = g_list_find_custom (contact_groups,
-						group_name,
-						(GCompareFunc) strcmp);
-		if (to_remove) {
-			gossip_debug (DEBUG_DOMAIN, "Contact %d removed from group '%s'",
-				      gossip_contact_get_handle (contact),
-				      group_name);
-			contact_groups = g_list_remove_link (contact_groups,
-							     to_remove);
-			tp_contact_list_block_contact (list, contact);
-			gossip_contact_set_groups (contact, contact_groups);
-			tp_contact_list_unblock_contact (list, contact);
-		}
-
-		g_list_free (contact_groups);
+		tp_contact_list_block_contact (list, contact);
+		gossip_contact_remove_group (contact, group_name);
+		tp_contact_list_unblock_contact (list, contact);
 
 		g_object_unref (contact);
 	}
@@ -1519,11 +1516,16 @@ tp_contact_list_group_members_removed_cb (GossipTelepathyGroup *group,
 }
 
 static void
-tp_contact_list_get_contacts_foreach (guint           handle,
+tp_contact_list_get_members_foreach (guint           handle,
 				      GossipContact  *contact,
 				      GList         **contacts)
 {
-	*contacts = g_list_append (*contacts, g_object_ref (contact));
+	GossipSubscription subscription;
+
+	subscription = gossip_contact_get_subscription (contact);
+	if (subscription & GOSSIP_SUBSCRIPTION_TO) {
+		*contacts = g_list_append (*contacts, g_object_ref (contact));
+	}
 }
 
 static void
@@ -1833,7 +1835,8 @@ tp_contact_list_status_changed_cb (MissionControl                  *mc,
 
 	account = mc_account_lookup (unique_name);
 	if (status != TP_CONN_STATUS_DISCONNECTED ||
-	    !gossip_account_equal (account, priv->account)) {
+	    !gossip_account_equal (account, priv->account) ||
+	    !priv->tp_conn) {
 		g_object_unref (account);
 		return;
 	}
